//===-- tysan_interface.h ---------------------------------------*- C++ -*-===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//
//
// This file is a part of TypeSanitizer.
//
// Public interface header for TySan.
//===----------------------------------------------------------------------===//
#ifndef SANITIZER_TYSAN_INTERFACE_H
#define SANITIZER_TYSAN_INTERFACE_H

#include <sanitizer/common_interface_defs.h>
#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif

// No interfaces currently

#ifdef __cplusplus
}
#endif

#endif
