/*===---- x86intrin.h - X86 intrinsics -------------------------------------===
 *
 * Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
 * See https://llvm.org/LICENSE.txt for license information.
 * SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 *
 *===-----------------------------------------------------------------------===
 */

#ifndef __X86INTRIN_H
#define __X86INTRIN_H

#include <ia32intrin.h>

#include <immintrin.h>

#include <prfchwintrin.h>

#include <ammintrin.h>

#include <fma4intrin.h>

#include <xopintrin.h>

#include <tbmintrin.h>

#include <lwpintrin.h>

#include <mwaitxintrin.h>

#include <clzerointrin.h>

#include <rdpruintrin.h>

#endif /* __X86INTRIN_H */
